'''
Created on Feb 18, 2014

@author: sean
'''
from __future__ import unicode_literals
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
import unittest
from mock import patch

class Test(CLITestCase):
    
    @patch('binstar_client.commands.login.store_token')
    @patch('getpass.getpass')
    @patch('binstar_client.commands.login.input')
    @urlpatch
    def test_login(self, urls, input, getpass, store_token):
        input.return_value = 'test_user'
        getpass.return_value = 'password'
        
        auth = urls.register(method='POST', path='/authentications', content='{"token": "a-token"}')
        main(['--show-traceback', 'login'], False)
        self.assertIn('login successful', self.stream.getvalue()) 
        
        auth.assertCalled()
        
        self.assertIn('Authorization', auth.req.headers)
        self.assertIn('Basic ', auth.req.headers['Authorization'])
        
        store_token.assert_called_once()
        self.assertEqual(store_token.call_args[0][0], 'a-token')
        
        
if __name__ == '__main__':
    unittest.main()



'''
Copy packages from one account to another
'''
from __future__ import unicode_literals, print_function
from binstar_client.utils import get_binstar, parse_specs
import logging
from binstar_client import errors

log = logging.getLogger('binstar.whoami')

def main(args):
    bs = get_binstar(args)

    spec = args.spec
    channels = bs.list_channels(spec.user)
    if args.from_channel not in channels:
        raise errors.UserError("Channel %s does not exist\n\tplease choose from: %s" % (args.from_channel, ', '.join(channels)))
    files = bs.copy(spec.user, spec.package, spec.version, spec._basename,
                    to_owner=args.to_owner, from_channel=args.from_channel, to_channel=args.to_channel)
    for binstar_file in files:
        print("Copied file: %(basename)s" % binstar_file)

    if files:
        log.info("Copied %i files" % len(files))
    else:
        log.warning("Did not copy any files. Please check your inputs with\n\n\tbinstar show %s" % spec)


def add_parser(subparsers):
    parser = subparsers.add_parser('copy',
                                      help='Copy packages from one account to another',
                                      description=__doc__)

    parser.add_argument('spec', help=('Package - written as user/package/version[/filename] '
                                      'If filename is not given, copy all files in the version'), type=parse_specs)
    parser.add_argument('--to-owner', help='User account to copy package to (default: your account)')
    parser.add_argument('--from-channel', help='Channel to copy packages from', default='main')
    parser.add_argument('--to-channel', help='Channel to put all packages into', default='main')
    parser.set_defaults(main=main)




'''
Manage your binstar channels

'''
from __future__ import unicode_literals, print_function
from binstar_client.utils import get_binstar
import logging
import argparse

log = logging.getLogger('binstar.channel')

def main(args):
    binstar = get_binstar(args)
    if args.organization:
        owner = args.organization
    else:
        current_user = binstar.user()
        owner = current_user['login']

    if args.copy:
        binstar.copy_channel(args.copy[0], owner, args.copy[1])
        log.info("Copied channel %s to %s" % tuple(args.copy))
    if args.remove:
        binstar.remove_channel(args.remove, owner)
        log.info("Removed channel %s" % args.remove)
    elif args.list:
        log.info('Channels')
        for channel, info in binstar.list_channels(owner).items():
            if isinstance(info, int):  # OLD API
                log.info((' + %s ' % channel))
            else:
                log.info((' + %s ' % channel) + ('[locked]' if info['is_locked'] else ''))

    elif args.show:
        info = binstar.show_channel(args.show, owner)
        log.info(('Channel %s ' % args.show) + ('[locked]' if info['is_locked'] else ''))
        for f in info['files']:
            log.info('  + %(full_name)s' % f)
    elif args.lock:
        binstar.lock_channel(args.lock, owner)
        log.info("Channel %s is now locked" % args.lock)
    elif args.unlock:
        binstar.unlock_channel(args.unlock, owner)
        log.info("Channel %s is now unlocked" % args.unlock)
    else:
        raise NotImplementedError()

def add_parser(subparsers):
    subparser = subparsers.add_parser('channel',
                                      help='Manage your binstar channels',
                                      formatter_class=argparse.RawDescriptionHelpFormatter,
                                      description=__doc__)

    subparser.add_argument('-o', '--organization',
                           help="Manage an organizations channels")

    group = subparser.add_mutually_exclusive_group(required=True)
    group.add_argument('--copy', nargs=2, metavar='CHANNEL')
    group.add_argument('--list', action='store_true',
                       help="list all channels for a user")
    group.add_argument('--show', metavar='CHANNEL',
                       help="Show all of the files in a channel")
    group.add_argument('--lock', metavar='CHANNEL',
                       help="Lock a channel")
    group.add_argument('--unlock', metavar='CHANNEL',
                       help="Unlock a channel")

    group.add_argument('--remove', metavar='CHANNEL',
                       help="Remove a channel")

    subparser.set_defaults(main=main)



from __future__ import print_function

import json
from os import path
from pprint import pprint
import sys
import tarfile


arch_map = {('osx', 'x86_64'):'osx-64',
            ('osx', 'x86'):'osx-32',
            ('win', 'x86'):'win-32',
            ('win', 'x86_64'):'win-64',
            ('linux', 'x86'):'linux-32',
            ('linux', 'x86_64'):'linux-64',
            (None, None): 'any-any',
           }

os_map = {'osx':'darwin', 'win':'win32'}


def transform_conda_deps(deps):
    """
    Format dependencies into a common binstar format
    """
    depends = {}
    for dep in deps:
        dep = dep.strip()
        name_spec = dep.split(' ', 1)
        if len(name_spec) == 1:
            name, = name_spec
            depends[name] = []
        elif len(name_spec) == 2:
            name, spec = name_spec
            if spec.endswith('*'):  # Star does nothing in semver
                spec = spec[:-1]
            depends[name] = [['==', spec]]
        elif len(name_spec) == 3:
            name, spec, build_str = name_spec
            if spec.endswith('*'):  # Star does nothing in semver
                spec = spec[:-1]
            depends[name] = [['==', '%s+%s' % (spec, build_str)]]

    return {'depends': depends,
            'depends_index': list(depends.keys()), }



def inspect_conda_package(filename, fileobj):

    tar = tarfile.open(filename, fileobj=fileobj)
    index = tar.extractfile('info/index.json')
    index = json.loads(index.read().decode())

    try:
        recipe = tar.extractfile('info/recipe.json')
        recipe = json.loads(recipe.read().decode())
    except KeyError:
        recipe = {}

    about = recipe.pop('about', {})

    os_arch = arch_map[(index['platform'], index['arch'])]
    machine = index['arch']
    operatingsystem = os_map.get(index['platform'], index['platform'])

    package_data = {
                    'name': index.pop('name'),
                    'summary': about.get('summary', ''),
                    'license': about.get('license'),
                    }
    release_data = {
                    'version': index.pop('version'),
                    'home_page': about.get('home'),
                    'description': '',
                    }
    file_data = {
                'basename': '%s/%s' % (os_arch, path.basename(filename)),
                'attrs':{
                        'operatingsystem': operatingsystem,
                        'machine': machine,
                        'target-triplet': '%s-any-%s' % (machine, operatingsystem)
                         }
                 }

    file_data['attrs'].update(index)
    file_data['dependencies'] = transform_conda_deps(index['depends'])
    return package_data, release_data, file_data

def main():
    filename = sys.argv[1]
    with open(filename) as fileobj:
        package_data, release_data, file_data = inspect_conda_package(filename, fileobj)
    pprint(package_data)
    print('--')
    pprint(release_data)
    print('--')
    pprint(file_data)

if __name__ == '__main__':
    main()



'''
Created on Feb 18, 2014

@author: sean
'''
from __future__ import unicode_literals
import unittest
import mock
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
import json


class Test(CLITestCase):
    
        
    @urlpatch
    def test_whoami_anon(self, urls):
        
        user = urls.register(method='GET', path='/user', status=401)
        
        main(['--show-traceback', 'whoami'], False)
        self.assertIn('Anonymous User', self.stream.getvalue())
        
        user.assertCalled() 
        
    @urlpatch
    def test_whoami(self, urls):
        
        content = json.dumps({'login': 'eggs', 'created_at':'1/2/2000'})
        user = urls.register(method='GET', path='/user', content=content)
         
        main(['--show-traceback', 'whoami'], False)
        self.assertIn('eggs', self.stream.getvalue()) 

        user.assertCalled() 
        
        
if __name__ == '__main__':
    unittest.main()



'''
Register a package in binstar.

This command must be run before 'binstar upload' if the package namespace does
not exist.

eg:
    binstar register CONDA_PACKAGE_1.bz2
    binstar upload CONDA_PACKAGE_1.bz2
'''
from __future__ import unicode_literals
from binstar_client import BinstarError, NotFound
from binstar_client.errors import UserError
from binstar_client.utils import get_binstar, get_config
from binstar_client.utils.detect import detect_package_type, get_attrs
from os.path import exists
import argparse
import logging
import sys

log = logging.getLogger('binstar.register')


def main(args):

    binstar = get_binstar(args)

    if args.user:
        username = args.user
    else:
        user = binstar.user()
        username = user ['login']

    if not exists(args.filename):
        raise BinstarError('file %s does not exist' % (args.filename))

    log.info('detecting package type ...')
    sys.stdout.flush()
    package_type = detect_package_type(args.filename)
    if package_type is None:
        raise UserError('Could not detect package type of file %r' % args.filename)

    log.info(package_type)

    log.info('extracting package attributes ...')
    sys.stdout.flush()
    try:
        package_attrs, _, _ = get_attrs(package_type, args.filename)
    except Exception:
        if args.show_traceback:
            raise

        raise BinstarError('Trouble reading metadata from %r. Please make sure this package is correct.' % (args.filename))

    if args.summary:
        summary = args.summary
    else:
        summary = package_attrs['summary']

    if args.package:
        if package_attrs['name'].lower() != args.package.lower():
            raise BinstarError('Package name on the command line does not match the package name in the file "%s"' % args.filename)
        package_name = args.package
    else:
        package_name = package_attrs['name']

    try:
        binstar.package(username, package_name)
    except NotFound:
        binstar.add_package(username, package_name,
                            summary,
                            package_attrs['license'],
                            public=args.access != 'private')
        log.info('Created package %s/%s' % (username, package_name))
    else:
        raise UserError('Package %s/%s already exists' % (username, package_name))


def add_parser(subparsers):

    config = get_config()

    parser = subparsers.add_parser('register',
                                      formatter_class=argparse.RawDescriptionHelpFormatter,
                                      help='Register a package on binstar',
                                      description=__doc__)

    parser.add_argument('filename', help='Inspect this file, to get the package name and summary', default=None)

    parser.add_argument('-u', '--user', help='User account, defaults to the current user')
    parser.add_argument('-p', '--package', help='Defaults to the packge name in the uploaded file')
    parser.add_argument('-s', '--summary', help='Summary of the package')

    perms = parser.add_mutually_exclusive_group()

    package_access = config.get('package_access', 'personal')
    perms.desciption = 'The package permissions'

    perms.add_argument('--private', action='store_const',
                       dest='access', const='private',
                       default=package_access == 'private',
                       help='Set the permissions of the package to private (if it does not exist)')

    perms.add_argument('--personal', action='store_const',
                       dest='access', const='personal',
                       default=package_access == 'personal',
                       help=('Set the permissions of the package to public. '
                             'Do not publish this to the global public repo. This package will be kept in you user repository.'))

    parser.set_defaults(main=main)



'''
Created on Feb 18, 2014

@author: sean
'''
from __future__ import unicode_literals
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
import unittest
from binstar_client import errors

class Test(CLITestCase):

    @urlpatch
    def test_upload_bad_package(self, registry):

        registry.register(method='GET', path='/user', content='{"login": "eggs"}')
        registry.register(method='GET', path='/package/eggs/foo', content='{}', status=404)
        registry.register(method='POST', path='/package/eggs/foo', content='{}', status=200)
        registry.register(method='GET', path='/release/eggs/foo/0.1', content='{}')
        registry.register(method='GET', path='/dist/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', status=404, content='{}')

        content = {"post_url": "http://s3_url.com/s3_url", "form_data": {}, "dist_id": "dist_id"}
        registry.register(method='POST', path='/stage/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', content=content)

        registry.register(method='POST', path='/s3_url', status=201)
        registry.register(method='POST', path='/commit/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', status=200, content={})

        main(['--show-traceback', 'upload', self.data_dir('foo-0.1-0.tar.bz2')], False)

        registry.assertAllCalled()


    @urlpatch
    def test_upload_bad_package_no_register(self, registry):

        registry.register(method='GET', path='/user', content='{"login": "eggs"}')
        registry.register(method='GET', path='/package/eggs/foo', status=404)

        with self.assertRaises(errors.UserError):
            main(['--show-traceback', 'upload', '--no-register', self.data_dir('foo-0.1-0.tar.bz2')], False)

        registry.assertAllCalled()

    @urlpatch
    def test_upload(self, registry):

        registry.register(method='GET', path='/user', content='{"login": "eggs"}')
        registry.register(method='GET', path='/package/eggs/foo', content='{}')
        registry.register(method='GET', path='/release/eggs/foo/0.1', content='{}')
        registry.register(method='GET', path='/dist/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', status=404, content='{}')

        content = {"post_url": "http://s3_url.com/s3_url", "form_data": {}, "dist_id": "dist_id"}
        registry.register(method='POST', path='/stage/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', content=content)

        registry.register(method='POST', path='/s3_url', status=201)
        registry.register(method='POST', path='/commit/eggs/foo/0.1/osx-64/foo-0.1-0.tar.bz2', status=200, content={})

        main(['--show-traceback', 'upload', self.data_dir('foo-0.1-0.tar.bz2')], False)

        registry.assertAllCalled()



if __name__ == '__main__':
    unittest.main()



'''
Print the information of the current user
'''
from __future__ import unicode_literals
from binstar_client import errors
from binstar_client.utils import get_binstar
from binstar_client.utils.pprint import pprint_user
import logging

log = logging.getLogger('binstar.whoami')

def main(args):
    binstar = get_binstar(args)

    try:
        user = binstar.user()
    except errors.Unauthorized:
        log.info('Anonymous User')
        return 1

    pprint_user(user)


def add_parser(subparsers):
    subparser = subparsers.add_parser('whoami',
                                      help='Print the information of the current user',
                                      description=__doc__)

    subparser.set_defaults(main=main)



'''
Binstar configuration

Get, Set, Remove or Show the binstar configuration.


'''
from __future__ import print_function

from ast import literal_eval
import logging
from pprint import pprint

from binstar_client.errors import ShowHelp
from binstar_client.utils import get_config, set_config, SITE_CONFIG, \
    USER_CONFIG

log = logging.getLogger('binstar.config')

def try_eval(item):
    try:
        return literal_eval(item)
    except SyntaxError:
        return item

def recursive_set(config_data, key, value, ty):
    while '.' in key:
        prefix, key = key.split('.', 1)
        config_data = config_data.setdefault(prefix, {})

    config_data[key] = ty(value)

def recursive_remove(config_data, key):
    while '.' in key:
        if not config_data:
            return
        prefix, key = key.split('.', 1)
        config_data = config_data.get(prefix, {})

    del config_data[key]



def main(args):
    config = get_config()

    if args.show:
        fmt = ' + %s: %r'
        log.info('Site Config: %s' % SITE_CONFIG)
        for key_value in get_config(user=False).items():
            log.info(fmt % key_value)
        log.info("")
        log.info('User Config: %s' % USER_CONFIG)
        for key_value in get_config(site=False).items():
            log.info(fmt % key_value)
        log.info("")
        return

    if args.get:
        log.info(config[args.get])
        return

    if args.files:
        log.info('User Config: %s' % USER_CONFIG)
        log.info('Site Config: %s' % SITE_CONFIG)
        return

    config = get_config(args.user, not args.user)

    for key, value in args.set:
        recursive_set(config, key, value, args.type)
        config[key] = args.type(value)

    for key in args.remove:
        try:
            recursive_remove(config, key)
        except KeyError:
            log.error("Key %s does not exist" % key)

    if not (args.set or args.remove):
        raise ShowHelp()

    set_config(config, args.user)


def add_parser(subparsers):
    parser = subparsers.add_parser('config',
                                      help='Binstar configuration',
                                      description=__doc__)

    parser.add_argument('--type', type=try_eval, default=str, help='The type of the values in the set commands')
    parser.add_argument('--set', nargs=2, action='append', default=[], help='sets a new variable: name value', metavar=('name', 'value'))
    parser.add_argument('--get', help='get value: name', metavar='name')
    parser.add_argument('--remove', action='append', default=[], help='removes a variable')
    parser.add_argument('--show', action='store_true', default=False, help='show all variables')
    parser.add_argument('-u', '--user', action='store_true', dest='user', default=True, help='set a variable for this user')
    parser.add_argument('-s', '--site', action='store_false', dest='user', help='set a variable for all users on this machine')
    parser.add_argument('-f', '--files', action='store_true', help='show the config file names')


    parser.set_defaults(main=main, sub_parser=parser)




'''
Upload a file to binstar. 

This command must be run after 'binstar register' if the package namespace does
not exist on binstar.

eg:
    binstar register CONDA_PACKAGE_1.bz2
    binstar upload CONDA_PACKAGE_1.bz2

'''
from __future__ import unicode_literals

import argparse
from glob import glob
import logging
import os
from os.path import exists
import sys

from binstar_client import BinstarError, NotFound, Conflict
from binstar_client.errors import UserError
from binstar_client.utils import get_binstar, bool_input, upload_print_callback
from binstar_client.utils.detect import detect_package_type, get_attrs


# Python 3 Support
try:
    input = raw_input
except NameError:
    input = input


log = logging.getLogger('binstar.upload')


def create_release(binstar, username, package_name, version, description, announce=None):
    binstar.add_release(username, package_name, version, [],
                        announce, description)

def create_release_interactive(binstar, username, package_name, version):

    log.info('\nThe release %s/%s/%s does not exist' % (username, package_name, version))
    if not bool_input('Would you like to create it now?'):
        log.info('good-bye')
        raise SystemExit(-1)

    description = input('Enter a short description of the release:\n')
    log.info("\nAnnouncements are emailed to your package followers.")
    make_announcement = bool_input('Would you like to make an announcement to the package followers?', False)
    if make_announcement:
        announce = input('Markdown Announcement:\n')
    else:
        announce = ''

    binstar.add_release(username, package_name, version, [],
                        announce, description)

def determine_package_type(filename, args):
    """
    return the file type from the inspected package or from the 
    -t/--package-type argument
    """
    if args.package_type:
        package_type = args.package_type
    else:
        log.info('detecting package type ...')
        sys.stdout.flush()
        package_type = detect_package_type(filename)
        if package_type is None:
            raise BinstarError('Could not detect package type of file %r please specify package type with option --package-type' % filename)
        log.info(package_type)

    return package_type

def get_package_name(args, package_attrs, filename, package_type):
    if args.package:
        if 'name' in package_attrs and package_attrs['name'].lower() != args.package.lower():
            raise BinstarError('Package name on the command line does not match the package name in the file "%s"' % filename)
        package_name = args.package
    else:
        if 'name' not in package_attrs:
            raise BinstarError("Could not detect package name for package type %s, please use the --package option" % (package_type,))
        package_name = package_attrs['name']

    return package_name


def get_version(args, release_attrs, package_type):
    if args.version:
        version = args.version
    else:
        if 'version' not in release_attrs:
            raise BinstarError("Could not detect package version for package type %s, please use the --version option" % (package_type,))
        version = release_attrs['version']
    return version

def add_package(binstar, args, username, package_name, package_attrs, package_type):
    try:
        binstar.package(username, package_name)
    except NotFound:
        if args.no_register:
            raise UserError('Binstar package %s/%s does not exist. '
                            'Please run "binstar register" to create this package namespace in the cloud.' % (username, package_name))
        else:

            if args.summary:
                summary = args.summary
            else:
                if 'summary' not in package_attrs:
                    raise BinstarError("Could not detect package summary for package type %s, please use the --summary option" % (package_type,))
                summary = package_attrs['summary']

            binstar.add_package(username, package_name, summary, package_attrs.get('license'),
                                public=True)


def add_release(binstar, args, username, package_name, version, release_attrs):
    try:
        binstar.release(username, package_name, version)
    except NotFound:
        if args.mode == 'interactive':
            create_release_interactive(binstar, username, package_name, version)
        else:
            create_release(binstar, username, package_name, version, release_attrs['description'])


def remove_existing_file(binstar, args, username, package_name, version, file_attrs):
    try:
        binstar.distribution(username, package_name, version, file_attrs['basename'])
    except NotFound:
        return False
    else:
        if args.mode == 'force':
            log.warning('Distribution %s already exists ... removing' % (file_attrs['basename'],))
            binstar.remove_dist(username, package_name, version, file_attrs['basename'])
        if args.mode == 'interactive':
            if bool_input('Distribution %s already exists. Would you like to replace it?' % (file_attrs['basename'],)):
                binstar.remove_dist(username, package_name, version, file_attrs['basename'])
            else:
                log.info('Not replacing distribution %s' % (file_attrs['basename'],))
                return True


def main(args):

    binstar = get_binstar(args)

    if args.user:
        username = args.user
    else:
        user = binstar.user()
        username = user ['login']

    uploaded_packages = []

    # Flatten file list because of 'windows_glob' function
    files = [f for fglob in args.files for f in fglob]

    for filename in files:

        if not exists(filename):
            raise BinstarError('file %s does not exist' % (filename))

        package_type = determine_package_type(filename, args)

        log.info('extracting package attributes for upload ...')
        sys.stdout.flush()
        try:
            package_attrs, release_attrs, file_attrs = get_attrs(package_type, filename)
        except Exception:
            if args.show_traceback:
                raise
            raise BinstarError('Trouble reading metadata from %r. Is this a valid %s package' % (filename, package_type))

        if args.build_id:
            file_attrs['attrs']['binstar_build'] = args.build_id


        log.info('done')

        package_name = get_package_name(args, package_attrs, filename, package_type)
        version = get_version(args, release_attrs, package_type)


        add_package(binstar, args, username, package_name, package_attrs, package_type)

        add_release(binstar, args, username, package_name, version, release_attrs)


        with open(filename, 'rb') as fd:
            log.info('\nUploading file %s/%s/%s/%s ... ' % (username, package_name, version, file_attrs['basename']))
            sys.stdout.flush()

            if remove_existing_file(binstar, args, username, package_name, version, file_attrs):
                continue
            try:
                upload_info = binstar.upload(username, package_name, version, file_attrs['basename'],
                                             fd, package_type,
                                             args.description,
                                             dependencies=file_attrs.get('dependencies'),
                                             attrs=file_attrs['attrs'],
                                             channels=args.channels,
                                             callback=upload_print_callback(args))
            except Conflict:
                full_name = '%s/%s/%s/%s' % (username, package_name, version, file_attrs['basename'])
                log.info('Distribution already exists. Please use the -i/--interactive or --force options or `binstar remove %s`' % full_name)
                raise

            uploaded_packages.append([package_name, upload_info])
            log.info("\n\nUpload(s) Complete\n")


    for package, upload_info in uploaded_packages:
        package_url = upload_info.get('url', 'https://binstar.org/%s/%s' % (username, package))
        log.info("Package located at:\n%s\n" % package_url)


def windows_glob(item):
    if os.name == 'nt' and '*' in item:
        return glob(item)
    else:
        return [item]

def add_parser(subparsers):

    parser = subparsers.add_parser('upload',
                                   formatter_class=argparse.RawDescriptionHelpFormatter,
                                   help='Upload a file to binstar',
                                   description=__doc__)

    parser.add_argument('files', nargs='+', help='Distributions to upload', default=[], type=windows_glob)

    parser.add_argument('-c', '--channel', action='append', default=[], dest='channels',
                        help='Add this file to a specific channel. Warning: if the file Channels do not include "main", the file will not show up in your user channel')

    parser.add_argument('-u', '--user', help='User account, defaults to the current user')
    parser.add_argument('--no-progress', help="Don't show upload progress", action='store_true')
    parser.add_argument('-p', '--package', help='Defaults to the packge name in the uploaded file')
    parser.add_argument('-v', '--version', help='Defaults to the packge version in the uploaded file')
    parser.add_argument('-s', '--summary', help='Set the summary of the package')
    parser.add_argument('-t', '--package-type', help='Set the package type, defaults to autodetect')
    parser.add_argument('-d', '--description', help='description of the file(s)')

    parser.add_argument("--no-register", action="store_true", default=False)
    parser.add_argument('--build-id', help='Binstar-Build ID (internal only)')

    group = parser.add_mutually_exclusive_group()
    group.add_argument('-i', '--interactive', action='store_const', help='Run an interactive prompt if any packages are missing',
                        dest='mode', const='interactive')
    group.add_argument('-f', '--fail', help='Fail if a package or release does not exist (default)',
                                        action='store_const', dest='mode', const='fail')
    group.add_argument('--force', help='Force a package upload regardless of errors',
                                        action='store_const', dest='mode', const='force')

    parser.set_defaults(main=main)



import base64
import collections
import getpass
from hashlib import md5
import json
import logging
import os
from os.path import exists, join, dirname, expanduser, isfile, isdir
import stat
import sys
import time

from binstar_client.utils import appdirs
import yaml

from ..errors import UserError


try:
    import urlparse
    from urllib import quote_plus
except ImportError:
    from urllib import parse as urlparse
    from urllib.parse import quote_plus


try:
    input = raw_input
except NameError:
    input = input

log = logging.getLogger('binstar')

def jencode(*E, **F):
    payload = dict(*E, **F)
    return json.dumps(payload), {'Content-Type': 'application/json'}

from pkg_resources import parse_version as pv

class PackageSpec(object):
    def __init__(self, user, package=None, version=None, basename=None, attrs=None, spec_str=None):
        self._user = user
        self._package = package
        self._version = version
        self._basename = basename
        self.attrs = attrs
        if spec_str:
            self.spec_str = spec_str
        else:
            spec_str = str(user)
            if package:
                spec_str = '%s/%s' % (spec_str, package)
            if version:
                spec_str = '%s/%s' % (spec_str, version)
            if basename:
                spec_str = '%s/%s' % (spec_str, basename)
            self.spec_str = spec_str



    def __str__(self):
        return self.spec_str

    def __repr__(self):
        return '<PackageSpec %r>' % (self.spec_str)

    @property
    def user(self):
        if self._user is None:
            raise UserError('user not given (got %r expected <username> )' % (self.spec_str,))
        return self._user

    @property
    def name(self):
        if self._package is None:
            raise UserError('package not given in spec (got %r expected <username>/<package> )' % (self.spec_str,))
        return self._package

    @property
    def package(self):
        if self._package is None:
            raise UserError('package not given in spec (got %r expected <username>/<package> )' % (self.spec_str,))
        return self._package

    @property
    def version(self):
        if self._version is None:
            raise UserError('version not given in spec (got %r expected <username>/<package>/<version> )' % (self.spec_str,))
        return self._version

    @property
    def basename(self):
        if self._basename is None:
            raise UserError('basename not given in spec (got %r expected <username>/<package>/<version>/<filename> )' % (self.spec_str,))
        return self._basename

def package_specs(spec):
    user = spec
    package = None
    attrs = {}
    if '/' in user:
        user, package = user.split('/', 1)
    if '/' in package:
        raise TypeError('invalid package spec')

    return PackageSpec(user, package, None, None, attrs, spec)

def parse_specs(spec):
    user = spec
    package = version = basename = None
    attrs = {}
    if '/' in user:
        user, package = user.split('/', 1)
    if package and '/' in package:
        package, version = package.split('/', 1)

    if version and '/' in version:
        version, basename = version.split('/', 1)

    if basename and '?' in basename:
        basename, qsl = basename.rsplit('?', 1)
        attrs = dict(urlparse.parse_qsl(qsl))

    return PackageSpec(user, package, version, basename, attrs, spec)

def load_token(url):
    data_dir = appdirs.user_data_dir('binstar', 'ContinuumIO')
    tokenfile = join(data_dir, '%s.token' % quote_plus(url))
    if isfile(tokenfile):
        with open(tokenfile) as fd:
            token = fd.read()
    else:
        token = None
    return token

def get_binstar(args=None, cls=None):
    if not cls:
        from binstar_client import Binstar
        cls = Binstar
    config = get_config(remote_site=args and args.site)
    url = config.get('url', DEFAULT_URL)
    if getattr(args, 'log_level', 0) >= logging.INFO:
        sys.stderr.write("Using binstar api site %s\n" % url)
    if args and args.token:
        token = args.token
    else:
        token = load_token(url)

    return cls(token, domain=url,)

def store_token(token, args):
    config = get_config(remote_site=args and args.site)

    url = config.get('url', DEFAULT_URL)

    data_dir = appdirs.user_data_dir('binstar', 'ContinuumIO')
    if not isdir(data_dir):
        os.makedirs(data_dir)
    tokenfile = join(data_dir, '%s.token' % quote_plus(url))

    if isfile(tokenfile):
        os.unlink(tokenfile)
    with open(tokenfile, 'w') as fd:
        fd.write(token)
    os.chmod(tokenfile, stat.S_IRUSR)


def remove_token(args):
    config = get_config(remote_site=args and args.site)
    url = config.get('url', DEFAULT_URL)
    data_dir = appdirs.user_data_dir('binstar', 'ContinuumIO')
    tokenfile = join(data_dir, '%s.token' % quote_plus(url))

    if isfile(tokenfile):
        os.unlink(tokenfile)

def load_config(config_file):
    if exists(config_file):
        with open(config_file) as fd:
            data = yaml.load(fd)
            if data:
                return data

    return {}

SITE_CONFIG = join(appdirs.site_data_dir('binstar', 'ContinuumIO'), 'config.yaml')
USER_CONFIG = join(appdirs.user_data_dir('binstar', 'ContinuumIO'), 'config.yaml')
USER_LOGDIR = appdirs.user_log_dir('binstar', 'ContinuumIO')

def recursive_update(d, u):
    for k, v in u.items():
        if isinstance(v, collections.Mapping):
            r = recursive_update(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d

DEFAULT_URL = 'https://api.binstar.org'
ALPHA_URL = 'http://api.alpha.binstar.org'
DEFAULT_CONFIG = {
                  'sites': {'binstar': {'url': DEFAULT_URL},
                            'alpha': {'url': ALPHA_URL},
                            }
                  }

def get_config(user=True, site=True, remote_site=None):
    config = DEFAULT_CONFIG.copy()
    if site:
        recursive_update(config, load_config(SITE_CONFIG))
    if user:
        recursive_update(config, load_config(USER_CONFIG))

    remote_site = remote_site or config.get('default_site')
    sites = config.get('sites', {})

    if remote_site:
        if remote_site not in sites:
            raise UserError("Remote site alias %s does not exist in the config file" % remote_site)

        recursive_update(config, sites.get(remote_site, {}))

    return config

def set_config(data, user=True):

    config_file = USER_CONFIG if user else SITE_CONFIG

    data_dir = dirname(config_file)
    if not exists(data_dir):
        os.makedirs(data_dir)

    with open(config_file, 'w') as fd:
        yaml.dump(data, fd)



def compute_hash(fp, buf_size=8192, size=None, hash_algorithm=md5):
    hash_obj = hash_algorithm()
    spos = fp.tell()
    if size and size < buf_size:
        s = fp.read(size)
    else:
        s = fp.read(buf_size)
    while s:
        hash_obj.update(s)
        if size:
            size -= len(s)
            if size <= 0:
                break
        if size and size < buf_size:
            s = fp.read(size)
        else:
            s = fp.read(buf_size)
    hex_digest = hash_obj.hexdigest()

    b64encode = getattr(base64, 'encodebytes', base64.encodestring)
    base64_digest = b64encode(hash_obj.digest())
    if base64_digest[-1] == '\n':
        base64_digest = base64_digest[0:-1]
    # data_size based on bytes read.
    data_size = fp.tell() - spos
    fp.seek(spos)
    return (hex_digest, base64_digest, data_size)


class upload_in_chunks(object):
    def __init__(self, fd, chunksize=1 << 13):
        self.fd = fd
        self.chunksize = chunksize
        self.totalsize = os.fstat(fd.fileno()).st_size
        self.readsofar = 0

    def __iter__(self):
        sys.stderr.write('Progress:\n')
        while True:
            data = self.fd.read(self.chunksize)
            if not data:
                sys.stderr.write("\n")
                break
            self.readsofar += len(data)
            percent = self.readsofar * 1e2 / self.totalsize
            sys.stderr.write("\r{percent:3.0f}%".format(percent=percent))
            yield data

    def __len__(self):
        return self.totalsize


def upload_with_progress(fd):
    it = upload_in_chunks(fd)
    IterableToFileAdapter(it)

class IterableToFileAdapter(object):
    def __init__(self, iterable):
        self.iterator = iter(iterable)
        self.length = len(iterable)

    def read(self, size=-1):  # TBD: add buffer for `len(data) > size` case
        return next(self.iterator, b'')

    def __len__(self):
        return self.length



def bool_input(prompt, default=True):
        default_str = '[Y|n]' if default else '[y|N]'
        while 1:
            inpt = input('%s %s: ' % (prompt, default_str))
            if inpt.lower() in ['y', 'yes'] and not default:
                return True
            elif inpt.lower() in ['', 'n', 'no'] and not default:
                return False
            elif inpt.lower() in ['', 'y', 'yes']:
                return True
            elif inpt.lower() in ['n', 'no']:
                return False
            else:
                sys.stderr.write('please enter yes or no\n')


def upload_print_callback(args):
    start_time = time.time()
    if args.no_progress or args.log_level > logging.INFO:
        return lambda curr, total: None

    def callback(curr, total):
        curr_time = time.time()
        time_delta = curr_time - start_time

        remain = total - curr
        if curr and remain:
            eta = 1.0 * time_delta / curr * remain / 60.0
        else:
            eta = 0

        curr_kb = curr // 1024
        total_kb = total // 1024
        perc = 100.0 * curr / total if total else 0

        msg = '\r uploaded %(curr_kb)i of %(total_kb)iKb: %(perc).2f%% ETA: %(eta).1f minutes'
        sys.stderr.write(msg % locals())
        sys.stderr.flush()
        if curr == total:
            sys.stderr.write('\n')

    return callback



'''
Search binstar for packages
'''
from binstar_client.utils import get_binstar
from binstar_client.utils.pprint import pprint_packages
import logging
log = logging.getLogger('binstar.search')

def search(args):

    binstar = get_binstar(args)

    log.info("Run 'binstar show <USER/PACKAGE>' to get more details:")
    packages = binstar.search(args.name, package_type=args.package_type)
    pprint_packages(packages)
    log.info("Found %i packages" % len(packages))


def add_parser(subparsers):
    parser1 = subparsers.add_parser('search',
                                      help='Search binstar',
                                      description=__doc__)
    parser1.add_argument('name', nargs=1, help='Search string')
    parser1.add_argument('-t', '--package-type', choices=['conda', 'pypi'],
                         help='only search for packages of this type')
    parser1.set_defaults(main=search)





'''
Authenticate a user
'''
from __future__ import unicode_literals

import getpass
import logging
import socket
import sys

from binstar_client import errors
from binstar_client.utils import get_config, get_binstar, store_token, \
    bool_input


log = logging.getLogger('binstar.login')

try:
    input = raw_input
except NameError:
    input = input

def interactive_get_token(args):
    bs = get_binstar(args)
    config = get_config(remote_site=args.site)

    url = config.get('url', 'https://api.binstar.org')

    token = None
    username = input('Username: ')

    auth_name = 'binstar_client:%s' % (socket.gethostname())
    password = None
    for _ in range(3):
        try:
            sys.stderr.write("%s's " % username)

            if password is None:
                password = getpass.getpass(stream=sys.stderr)

            token = bs.authenticate(username, password, auth_name, url,
                                    created_with=' '.join(sys.argv),
                                    fail_if_already_exists=True)
            break

        except errors.Unauthorized:
            log.error('Invalid Username password combination, please try again')
            password = None
            continue

        except errors.BinstarError as err:
            if err.args[1] == 400:
                log.error('It appears you are already logged in from host %s' % socket.gethostname())
                log.error('Logging in again will remove the previous token.')
                if bool_input("Would you like to continue"):
                    bs.remove_authentication(auth_name)
                    continue
                else:
                    raise


    if token is None:
        msg = ('Sorry. Please try again '
               '(go to https://binstar.org/account/forgot_password '
               'to reset your password)')
        raise errors.BinstarError(msg)

    return token

def interactive_login(args):
    token = interactive_get_token(args)
    store_token(token, args)
    log.info('login successful')

def main(args):
    interactive_login(args)

def add_parser(subparsers):
    subparser = subparsers.add_parser('login',
                                      help='Authenticate a user',
                                      description=__doc__)

    subparser.set_defaults(main=main)



'''
Created on Jul 15, 2013

@author: sean
'''
from clyent.errors import ClyentError

class BinstarError(ClyentError):

    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)

        if not hasattr(self, 'message'):
            self.message = args[0] if args else None

class Unauthorized(BinstarError):
    pass

class Conflict(BinstarError):
    pass

class NotFound(BinstarError, IndexError):

    def __init__(self, *args, **kwargs):
        BinstarError.__init__(self, *args, **kwargs)
        IndexError.__init__(self, *args, **kwargs)

        self.message = args[0]

class UserError(BinstarError):
    pass


class ShowHelp(BinstarError):
    pass


class NoMetadataError(BinstarError):
    pass



from __future__ import print_function

from email.parser import Parser
import json
from os import path
from os.path import basename, splitext
from pprint import pprint
import sys
import tarfile
import zipfile

from binstar_client import errors
from binstar_client.inspect_package.uitls import extract_first, pop_key
import pkg_resources
import re


def parse_requirement(line, deps, extras, extra):
    req = pkg_resources.Requirement.parse(line)
    if extra:
        extras[extra][req.key] = req.specs
    else:
        deps[req.key] = req.specs

def parse_requires_txt(requires_txt):
    deps = {}
    error = False
    extras = {}
    extra = None

    for line in requires_txt.split('\n'):
        line = line.strip()

        if not line: continue

        if line.startswith('[') and line.endswith(']'):
            extra = line[1:-1]
            if extra == 'all':
                extra = None
            else:
                extras.setdefault(extra, {})

            # Dont parse this requirement
            continue
        try:
            parse_requirement(line, deps, extras, extra)
        except ValueError as err:
            error = True

    return {'has_dep_errors': error, 'depends': deps, 'extras':extras,
            'depends_index': list(deps.keys()),
            'optional_depends_index': [d for extra in extras.values() for d in extra.keys()]
            }

def format_rqeuirements(requires):
    obj = {}
    for req in requires:
        req = req.strip()
        req_spec = req.split(' ', 1)
        if len(req_spec) == 1:
            obj[req] = []
        else:
            req, spec = req_spec
            spec = spec.strip()
            if spec[0] == '(': spec = spec[1:]
            if spec[-1] == ')': spec = spec[:-1]

            req = pkg_resources.Requirement.parse('%s %s' % (req, spec))
            obj[req.key] = req.specs

    return obj

def format_run_requires_metadata(run_requires):
    deps = {}
    extras = {}
    environments = {}
    for run_require in run_requires:
        extra = run_require.get('extra')
        env = run_require.get('environment')
        requires = run_require['requires']

        if env:
            obj = environments.setdefault(env, {})
        elif extra is None or extra == 'all':
            obj = deps
        else:
            obj = extras.setdefault(extra, {})

        obj.update(format_rqeuirements(requires))

    optional = []
    optional += [d for extra in extras.values() for d in extra.keys()]
    optional += [d for extra in environments.values() for d in extra.keys()]

    attrs = {'has_dep_errors': False, 'depends': deps, 'extra_depends':extras,
             'environment_depends': environments,
            'depends_index': list(deps.keys()),
            'optional_depends_index': optional,
             }

    return attrs

def format_requires_metadata(run_requires):
    deps = {}
    extras = {}
    environments = {}

    extras_re = re.compile('extra == [\'\"](.*)[\'\"]')
    for key, requirements in run_requires.items():
        is_extra = extras_re.match(key)
        if is_extra:
            extra = is_extra.groups()[0]
            if extra is None or extra == 'all':
                obj = deps
            else:
                obj = extras.setdefault(extra, {})

        else:
            obj = environments.setdefault(key, {})

        obj.update(format_rqeuirements(requirements))

    optional = []
    optional += [d for extra in extras.values() for d in extra.keys()]
    optional += [d for extra in environments.values() for d in extra.keys()]

    attrs = {'has_dep_errors': False, 'depends': deps, 'extra_depends':extras,
             'environment_depends': environments,
            'depends_index': list(deps.keys()),
            'optional_depends_index': optional,
             }

    return attrs

def format_wheel_json_metadata(data, filename, zipfile):
    package_data = {'name': pop_key(data, 'name'),
                    'summary': pop_key(data, 'summary', None),
                    'license': pop_key(data, 'license', None),
                    }
    description_doc = pop_key(data.get('document_names') or {}, 'description', None)

    if description_doc:
        description = extract_first(zipfile, '*.dist-info/%s' % description_doc)
    else:
        description = None

    release_data = {
                    'version': pop_key(data, 'version'),
                    'description': description,
                    'home_page': pop_key(data.get('project_urls', {}), 'Home', None)
                    }

    data.update({
                 'packagetype': 'bdist_wheel',
                 'python_version':'source',
                 })
    if data.get('run_requires', {}):
        dependencies = format_run_requires_metadata(data['run_requires'])
    else:
        dependencies = format_requires_metadata(data.get('requires', {}))
    file_data = {
                 'basename': path.basename(filename),
                 'attrs': data,
                 'dependencies': dependencies,
                 }

    return package_data, release_data, file_data


def inspect_pypi_package_whl(filename, fileobj):
    tf = zipfile.ZipFile(fileobj)

    data = extract_first(tf, '*.dist-info/metadata.json')
    if data is None:
        data = extract_first(tf, '*.dist-info/pydist.json')

    if data:
        package_data, release_data, file_data = format_wheel_json_metadata(json.loads(data), filename, tf)
    else:
        package_data, release_data, file_data = {}, {}, {}

    file_components = filename[:-4].split('-')
    if len(file_components) == 5:
        _, _, python_version, abi, platform = file_components
        build_no = 0
    elif len(file_components) == 6:
        _, _, build_no, python_version, abi, platform = file_components
    else:
        raise TypeError("Bad wheel package name")
    if platform == 'any': platform = None
    if abi == 'none': abi = None

    file_data.setdefault('attrs', {})
    file_data['attrs'].update(build_no=build_no, python_version=python_version, abi=abi,
                              packagetype='bdist_wheel')
    file_data.update(platform=platform)
    return package_data, release_data, file_data


def disutils_dependencies(config_items):
        requirements = [v for k, v in config_items if k == 'Requires']
        depends = format_rqeuirements(requirements)
        return {'depends':depends,
                'depends_index': list(depends.keys()),
                'extras': {},
                'has_dep_errors': False,
                'optional_depends_index': []
                }
def inspect_pypi_package_sdist(filename, fileobj):

    tf = tarfile.open(filename, fileobj=fileobj)

    data = extract_first(tf, '*.egg-info/PKG-INFO')

    distrubite = False
    if data is None:
        data = extract_first(tf, '*/PKG-INFO')
        distrubite = True
        if data is None:
            raise errors.NoMetadataError("Could not find *.egg-info/PKG-INFO file in pypi sdist")

    config_items = Parser().parsestr(data).items()
    attrs = dict(config_items)
    name = pop_key(attrs, 'Name', None)

    if name is None:
        name = filename.split('-', 1)[0]

    package_data = {'name': name,
                    'summary': pop_key(attrs, 'Summary', None),
                    'license': pop_key(attrs, 'License', None),
                    }
    release_data = {
                    'version': pop_key(attrs, 'Version'),
                    'description': pop_key(attrs, 'Description', None),
                    'home_page': pop_key(attrs, 'Home-page', None)

                    }
    file_data = {
                 'basename': basename(filename),
                 'attrs': attrs,
                 }


    if distrubite:  # Distrubite does not create dep files
        file_data.update(dependencies=disutils_dependencies(config_items))

    requires_txt = extract_first(tf, '*.egg-info/requires.txt')
    if requires_txt:
        file_data.update(dependencies=parse_requires_txt(requires_txt))

    attrs.update({
                 'packagetype': 'sdist',
                 'python_version':'source',
                 })



    return package_data, release_data, file_data


def inspect_pypi_package_egg(filename, fileobj):
    tf = zipfile.ZipFile(fileobj)

    data = extract_first(tf, 'EGG-INFO/PKG-INFO')
    if data is None:
        raise errors.NoMetadataError("Could not find EGG-INFO/PKG-INFO file in pypi sdist")

    attrs = dict(Parser().parsestr(data).items())

    package_data = {'name': pop_key(attrs, 'Name'),
                    'summary': pop_key(attrs, 'Summary', None),
                    'license': pop_key(attrs, 'License', None),
                    }
    release_data = {
                    'version': pop_key(attrs, 'Version'),
                    'description': pop_key(attrs, 'Description', None),
                    'home_page': pop_key(attrs, 'Home-page', None)
                    }
    file_data = {
                 'basename': basename(filename),
                 'attrs': attrs,
                 }

    requires_txt = extract_first(tf, 'EGG-INFO/requires.txt')
    if requires_txt:
        file_data.update(dependencies=parse_requires_txt(requires_txt))

    if len(filename.split('-')) == 4:
        _, _, python_version, platform = filename[:-4].split('-')
    else:
        python_version = 'source'
        platform = None


    file_data.update(platform=platform)

    attrs.update({
                 'packagetype': 'bdist_egg',
                 'python_version': python_version,
                 })

    return package_data, release_data, file_data


def inspect_pypi_package_zip(filename, fileobj):
    filename, fileobj

    tf = zipfile.ZipFile(fileobj)

    data = extract_first(tf, '*/PKG-INFO')
    if data is None:
        raise errors.NoMetadataError("Could not find EGG-INFO/PKG-INFO file in pypi sdist")

    attrs = dict(Parser().parsestr(data).items())
    package_data = {'name': pop_key(attrs, 'Name'),
                    'summary': pop_key(attrs, 'Summary', None),
                    'license': pop_key(attrs, 'License', None),
                    }
    release_data = {
                    'version': pop_key(attrs, 'Version'),
                    'description': pop_key(attrs, 'Description', None),
                    'home_page': pop_key(attrs, 'Home-page', None)
                    }
    file_data = {
                 'basename': basename(filename),
                 'attrs': attrs,
                 }

    attrs.update({
                 'packagetype': 'bdist_egg',
                 'python_version': 'source',
                 })

    return package_data, release_data, file_data

def inspect_pypi_package_exe(filename, fileobj):

    # ipython-0.12.1.win-amd64.exe
    name_version, windist = filename[:-4].rsplit('.', 1)

    name, version = name_version.split('-', 1)

    package_data = {'name': name}
    release_data = {version: version}
    file_data = {'attrs': {
                           'packagetype': 'bdist_wininst',
                           'python_version': 'source',
                           'windist': windist,
                           },
                 'basename': path.basename(filename),
                }

    return package_data, release_data, file_data

def inspect_pypi_package_rpm(filename, fileobj):
    # ipython-0.12.1.win-amd64.exe
    name_version, rpmarch = filename[:-4].rsplit('.', 1)

    name, version, python_version = name_version.split('-', 2)

    package_data = {'name': name}
    release_data = {version: version}
    file_data = {'attrs': {
                           'packagetype': 'bdist_rpm',
                           'python_version': python_version,
                           'rpmarch': rpmarch,
                           },
                 'basename': path.basename(filename),
                }
    return package_data, release_data, file_data


def inspect_pypi_package(filename, fileobj):
    if filename.endswith('.tar.gz') or filename.endswith('.tar.bz2'):
        return inspect_pypi_package_sdist(filename, fileobj)
    if filename.endswith('.whl'):
        return inspect_pypi_package_whl(filename, fileobj)
    if filename.endswith('.egg'):
        return inspect_pypi_package_egg(filename, fileobj)
    if filename.endswith('.zip'):
        return inspect_pypi_package_zip(filename, fileobj)
    if filename.endswith('.exe'):
        return inspect_pypi_package_exe(filename, fileobj)
    if filename.endswith('.rpm'):
        return inspect_pypi_package_rpm(filename, fileobj)

    _, etx = path.splitext(filename)
    raise errors.NoMetadataError("Can not inspect pypi package with file extension %s" % etx)

def main():
    filename = sys.argv[1]
    with open(filename) as fileobj:
        package_data, release_data, file_data = inspect_pypi_package(filename, fileobj)
    pprint(package_data)
    print('--')
    pprint(release_data)
    print('--')
    pprint(file_data)

if __name__ == '__main__':
    main()
